-- Camp Site reference plugin by David Down
-- coding: utf-8
import "Turbine.Gameplay"
import "Vinny.CampSite.CS_Data"
import "Vinny.Common.Help"

function print(text) Turbine.Shell.WriteLine("<rgb=#00FFFF>CS:</rgb> "..text) end
function printh(text) print("<rgb=#00FF00>"..text.."</rgb>") end
function printe(text) print("<rgb=#FF6040>Error: "..text.."</rgb>") end

local CSv = "Camp Site "..Plugins["CampSite"]:GetVersion()
local Coord = "^(%d+%.%d[NnSs]),(%d+%.%d[EeWw])$"
local Coord2 = "^(%d+%.%d[NS]), (%d+%.%d[EW])$"
local Zloc = "^(.+): .+: (.+)$"

player = Turbine.Gameplay.LocalPlayer.GetInstance()
plevel = player:GetLevel()
printh(CSv.." data initialized.")

CS_Opt = Turbine.PluginData.Load(Turbine.DataScope.Server,"CampSite_Opt")
if not CS_Opt then CS_Opt = {} end

import "Vinny.CampSite.CS_Window"

Plugins.CampSite.Open = function(sender,args)
	CS_window:SetVisible( true )
	CS_window:SetZOrder( 2 )
end

local function distance(dy,dx) return math.sqrt(dy*dy+dx*dx) end

local function locV(str,neg)
    local nbr = tonumber(str:sub(1,-2))
    if neg:find(str:sub(-1)) then nbr = -nbr end
    return nbr
end

function CS_Skills(table,skill)
	for i,t in ipairs(table) do
		local str,d = skill..t.n, ""
		if t.z=="zone" then str = t.n end
		if t.d then d = " ("..t.d..")" end
		print(string.format("%s, port to %s in %s%s",str,t.l,t.z,d))
		str = Train[t.tt]
		if t.td then str = t.td.." via "..str end
		if t.rr then str = str..", req: "..t.rr end
		print("  L:"..t.tl.." from "..str)
	end
end

function Find_loc(table,r,y,x)
	local d1,y1,x1,ln = 999,locV(y,"Ss"), locV(x,"Ww")
	local y2,x2,d
	for loc,t in pairs(table) do
		if t.r==r and t.f~=0 then
			y,x = t.l:match(Coord)
			if not y then printe("Bad Loc for "..loc) return end
			y2,x2 = locV(y,"Ss"), locV(x,"Ww")
			d = distance(y1-y2,x1-x2)
			if d<d1 then d1 = d; ln = loc end
		end
	end
	if ln==999 then return "Unknown" end
	d = string.format(" (%.1f units away).",d1)
	local t = table[ln]
	return ln.." @ "..t.l..d
end

CS_Command = Turbine.ShellCommand()

function CS_Command:Execute( cmd,args,zn )
	if cmd=="cs" and (args=="help" or args=="?") then
		Vinny.Common.Help(help,args)
		return
	end
	if cmd=="cs?" then
		Vinny.Common.Help(help,"??")
		return
	end
	if cmd=="csw" then
		CS_window:SetVisible( true )
		return
	end
	if cmd=="csl" then
		local reg,loc = args:match(Zloc)
		if loc then
			local r = Region[reg]
			if not r then printe("Unknown region: "..reg) return end
			local y,x = loc:match(Coord2)
			str = Find_loc(Camps,r,y,x)
			if str then print("The nearest camp site is "..str) end
			str = Find_loc(Rally,r,y,x)
			if str then print("The nearest rally point is "..str) end
		else printe("No location data in instance.") end
		return
	end
    local y,x = args:match(Coord)
	if cmd=="csr" then
		if Zones[args] then
			print("<rgb=#00FF00>Known rally points in "..args..":</rgb>")
			list,t,a = { }
			for name,t in pairs(Rally) do
				if t.z==args then table.insert(list,name) end
			end
			table.sort(list)
			for i,name in ipairs(list) do
				t = Rally[name]
				a = t.a and " ("..t.a..")" or ""
				print(name.." @ "..t.l..a)
			end
		elseif y then
			str = Find_loc(Rally,y,x)
			if str then print("The nearest rally point to "..args.." is "..str) end
		else printe("Unknown zone: "..args) end
		return
	end
	if args=="zones" then
		print("<rgb=#00FF00>Currently defined zones with camp sites:</rgb>")
		for i,name in ipairs(zones) do
			print(name.."("..Zones[name]..")")
		end
		return
	end
	if args=="horns" then
		local str = zn and " in "..zn or ""
		print("<rgb=#00FF00>Known locations with Mustering Horns"..str..":</rgb>")
		list,t = { }
		for name,t in pairs(Camps) do
			if t.h and (not zn or zn==t.z) then
				table.insert(list,name)
			end
		end
		table.sort(list)
		for i,name in ipairs(list) do
			t = Camps[name]
			print(string.format("%s @ %s in %s (%s), MH=L%d",name,t.l,t.z,t.d,t.h))
		end
		return
	end
	if args=="hunter" then
		print("<rgb=#00FF00>Hunter travel skills:</rgb>")
		CS_Skills(Hunter,"Guide to ")
		return
	end
	if args=="warden" then
		print("<rgb=#00FF00>Warden travel skills:</rgb>")
		CS_Skills(Warden,"Muster in ")
		return
	end
	if Zones[args] then
		print("<rgb=#00FF00>Known camp sites in "..args..":</rgb>")
		list,t,mh = { }
		for name,t in pairs(Camps) do
			if t.z==args and t.f~=0 then table.insert(list,name) end
		end
		table.sort(list)
		for i,name in ipairs(list) do
			t,mh = Camps[name], ""
			if t.h then mh = ", MH=L"..t.h
			elseif t.f then mh = ", L="..t.f end
			print(name.." @ "..t.l.." ("..t.d..")"..mh)
		end
		for i,t in pairs(Hunter) do
			if t.z==args then
				local str = t.d and " ("..t.d..")" or ""
				print("Hunters can guide to "..t.n.." @ "..t.l..str)
			end
		end
		return
	end
	local t = Camps[args]
	if t then
		local mh,z = "", t.z
		if t.h then mh = string.format(" (MH=%d)", t.h)
		elseif t.f then mh = string.format(" (L=%d)", t.f) end
		if t.a then z = t.a.." of "..z end
		print(string.format("%s @ %s in %s (%s)%s",args,t.l,z,t.d,mh))
		return
	end
    if y then
		str = Find_loc(Camps,y,x)
		if str then print("The nearest camp site to "..args.." is "..str) end
		return
	end
    Vinny.Common.Help(help,args)
end

Turbine.Shell.AddCommand( "cs;csr;csl;csw;cs?",CS_Command )

-- Options panel
import "Vinny.Common.Options"
Vinny.Common.Options_Init(print,CS_Opt,CS_window,"CampSite_Opt")

-- Help text
help = {
	pre = "CS",
	help = "Enter '/cs ?' for arguments, '/cs?' for commands",
	['?'] = "Unknown, enter '/cs help' or '/cs ?'",
	arg = {
		zones = "List known zones with camp sites.",
		horns = "List known locations with Mustering Horns.",
		hunter = "List Hunter 'Guide to' skills and requirements.",
		warden = "List Warden 'Muster in' skills and requirements.",
		["<zone>"] = "List camp sites in <zone>",
		["<name>"] = "List camp site <name> details",
	},
	cmd = {
		["csr <zone>"] = "List rally points in <zone>",
		["csl ;loc"] = "Display nearest campsite and rally point",
		csw = "Open Camp Site window.",
	},
}
